package com.galaxis.manatee.Test.zzx;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.galaxis.manatee.capsule.bo.ChuanyunObject;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunBuildProject;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunMaintenanceCycleDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunMaintenanceProDO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunFindAllBizDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeDeserializer;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeSerializer;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.CascadeType;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;
import javax.persistence.Transient;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@SpringBootTest
public class ProjectTest1 {
    @Autowired
    private ChuanYunManager chuanYunManager;

    @Test
    void test1() {  //将项目立项的数据同步到售后项目资料
        List<String> m = List.of("F0000016_2,维保", "F0000016_2,售后");
        var filter = Filter.instance(0, 500, true, Filter.OR, m);
        ChuanyunFindAllBizDTO<Object> chuanyunFindAllResponse = null;
        try {
            chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunBuildProject.SCHEMA_CODE, filter);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }

        if (chuanyunFindAllResponse.getReturnData() == null || chuanyunFindAllResponse.getReturnData().getBizObjectArray() == null) {
            return;
        }

        List<Object> bizObjectArray = chuanyunFindAllResponse.getReturnData().getBizObjectArray();
        ObjectMapper objectMapper = new ObjectMapper();
        List<ChuanyunBuildProject> bpList = objectMapper.convertValue(bizObjectArray, new TypeReference<>() {
        });

        for (int i = 0; i < bpList.size(); i++) {
            if (i > 2) {
                return;
            }

            ChuanyunBuildProject chuanyunBuildProject = bpList.get(i);
            // log.info("{chuanyunBuildProject => {}}", chuanyunBuildProject);

            try {
                Thread.sleep(200);  //避免同步数据时因为氚云的限制而导致失败

                Aftermarket aftermarket = new Aftermarket();
                aftermarket.setProjectName(chuanyunBuildProject.getProjectName());
                aftermarket.setProjectNo(chuanyunBuildProject.getProjectCode());
                aftermarket.setCustomName(chuanyunBuildProject.getCustomerName());
                aftermarket.setAfterProType(chuanyunBuildProject.getProjectType());
                aftermarket.setProClassify(chuanyunBuildProject.getProjectClassify());

                if (chuanyunBuildProject.getProjectType().equals("维保")) {    //DW(维保)
                    aftermarket.setMaintenanceProManager(chuanyunBuildProject.getProjectManager().getObjectId());
                    aftermarket.setMaintenanceProDirector(chuanyunBuildProject.getProjectDirector().getObjectId());
                    aftermarket.setMaintenanceSaleChargeMan(chuanyunBuildProject.getSalesDirector().getObjectId());
                    aftermarket.setImplProManager(chuanyunBuildProject.getImplementationManager().getObjectId());

                    //维保交付内容(子表)
                    List<AftermarketMaintenance> maintenanceProList = new ArrayList<>();
                    if (chuanyunBuildProject.getMaintenanceProList().size() > 0) {
                        for (int j = 0; j < chuanyunBuildProject.getMaintenanceProList().size(); j++) {
                            ChuanyunMaintenanceProDO maintenanceProDO = chuanyunBuildProject.getMaintenanceProList().get(j);
                            AftermarketMaintenance aftermarketMaintenance = new AftermarketMaintenance();
                            aftermarketMaintenance.setProProperty(maintenanceProDO.getProProperty());
                            aftermarketMaintenance.setProProperty(maintenanceProDO.getProContent());
                            aftermarketMaintenance.setProVersion(maintenanceProDO.getProVersion());
                            aftermarketMaintenance.setMaintenanceChargeMan(maintenanceProDO.getMaintenanceChargeMan().getObjectId());
                            aftermarketMaintenance.setRemark(maintenanceProDO.getRemark());
                            maintenanceProList.add(aftermarketMaintenance);
                        }
                    }
                    aftermarket.setMaintenanceProList(maintenanceProList);

                    //维保周期(子表)
                    List<AftermarketCycle> marketCycleList = new ArrayList<>();
                    if (chuanyunBuildProject.getMaintenanceCycleList().size() > 0) {
                        for (int k = 0; k < chuanyunBuildProject.getMaintenanceCycleList().size(); k++) {
                            ChuanyunMaintenanceCycleDO cycleDO = chuanyunBuildProject.getMaintenanceCycleList().get(k);
                            AftermarketCycle aftermarketCycle = new AftermarketCycle();
                            aftermarketCycle.setProProperty(cycleDO.getProProperty());
                            aftermarketCycle.setFreeMaintenanceMonth(cycleDO.getFreeMaintenanceMonth());
                            aftermarketCycle.setFreeMaintenanceStartDate(cycleDO.getFreeMaintenanceStartDate());
                            aftermarketCycle.setFreeMaintenanceEndDate(cycleDO.getFreeMaintenanceEndDate());
                            marketCycleList.add(aftermarketCycle);
                        }
                    }
                    aftermarket.setMaintenanceCycleList(marketCycleList);
                } else { //AS(售后)
                    aftermarket.setAfterSaleManager(chuanyunBuildProject.getProjectManager().getObjectId());
                    aftermarket.setAfterSaleDirector(chuanyunBuildProject.getProjectDirector().getObjectId());
                    aftermarket.setAfterSaleChargeMan(chuanyunBuildProject.getSalesDirector().getObjectId());

                    aftermarket.setDeliverManager(chuanyunBuildProject.getMainContents());
                    aftermarket.setSysMaintenanceMonth(chuanyunBuildProject.getSystemWarranty());
                    aftermarket.setDeviceMaintenanceMonth(chuanyunBuildProject.getDeviceWarranty());
                    aftermarket.setSetUpDate(chuanyunBuildProject.getSetUpDate());
                }

                // log.info("{aftermarket => {}}", aftermarket);

                String json = null;
                try {
                    json = objectMapper.writeValueAsString(aftermarket);
                    // log.info("{json => {}}", json);
                    chuanYunManager.save("D001789Srquo86tpfq1iuuqww4heu2w73", json, true);
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    @Data
    //售后项目资料的实体类
    static class Aftermarket {
        @JsonProperty("F0000001")
        private String projectName;                                         //项目名称

        @JsonProperty("F0000078")
        private String projectNo;                                           //项目编号

        @JsonProperty("F0000121")
        private String customName;                                          //客户名称

        @JsonProperty("F0000035")
        private String afterProType;                                        //售后项目类型

        @JsonProperty("F0000036")
        private String proClassify;                                         //项目分类

        @JsonProperty("F0000108")
        private String isDevOps;                                            //是否运维(仅维保项目有)

        @JsonProperty("F0000124")
        private String afterProCutoffDate;                                  //转售后工作移交日期(仅维保项目有)

        @JsonProperty("F0000104")
        private String maintenanceProManager;                               //维保项目经理(仅维保项目有)

        @JsonProperty("F0000105")
        private String maintenanceProDirector;                              //维保项目总监(仅维保项目有)

        @JsonProperty("F0000106")
        private String maintenanceSaleChargeMan;                            //销售负责人(仅针对维保项目)

        @JsonProperty("F0000107")
        private String implProManager;                                      //实施项目经理(仅维保项目有)

        @JsonProperty("D001789Fe8e95bd67f06467fb2e0d792a0555619")
        private List<AftermarketMaintenance> maintenanceProList;            //维保交付内容(子表)(仅维保项目有)

        @JsonProperty("D001789Fe75a894fabe3490ab47812cb7d79fd25")
        private List<AftermarketCycle> maintenanceCycleList;                //维保周期(子表)(仅维保项目有)

        @JsonProperty("F0000123")
        private String deliverManager;                                      //主要交付内容(仅售后项目有)

        @JsonProperty("F0000087")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        @DateTimeFormat(pattern = "yyyy/MM/dd HH:mm:ss")
        private LocalDateTime setUpDate;                                    //立项日期(仅售后项目有)

        @JsonProperty("F0000110")
        private String afterSaleManager;                                    //售后项目经理(仅售后项目有)

        @JsonProperty("F0000111")
        private String afterSaleDirector;                                   //售后项目总监(仅售后项目有)

        @JsonProperty("F0000112")
        private String afterSaleChargeMan;                                  //销售负责人(仅针对售后项目)

        @JsonProperty("F0000125")
        private Integer sysMaintenanceMonth;                                //系统免费维保期(仅针对售后项目)

        @JsonProperty("F0000128")
        private Integer deviceMaintenanceMonth;                             //设备免费维保期(仅针对售后项目)
    }

    @Data
    //维保交付内容(子表)(仅维保项目有)
    static class AftermarketMaintenance {
        //属性
        @JsonProperty("F0000080")
        private String proProperty;

        //交付内容
        @JsonProperty("F0000044")
        private String proContent;

        //系统版本
        @JsonProperty("F0000134")
        private String proVersion;

        //维保承接负责人
        @JsonProperty("F0000081")
        private String maintenanceChargeMan;

        //备注
        @JsonProperty("F0000046")
        private String remark;
    }

    @Data
    //维保周期(子表)(仅维保项目有)
    static class AftermarketCycle {
        //属性
        @JsonProperty("F0000100")
        private String proProperty;

        //免费维保月
        @JsonProperty("F0000101")
        private String freeMaintenanceMonth;

        //免费维保开始日期
        @JsonProperty("F0000102")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        @DateTimeFormat(pattern = "yyyy/MM/dd HH:mm:ss")
        private LocalDateTime freeMaintenanceStartDate;

        //免费维保结束日期
        @JsonProperty("F0000103")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        @DateTimeFormat(pattern = "yyyy/MM/dd HH:mm:ss")
        private LocalDateTime freeMaintenanceEndDate;

        //备注
        // @JsonProperty("F0000093")
        // private String remark;
    }
}

